@@ -2,11 +2,13 @@
 
 pub use self::iter_statistics::*;
 pub use self::order_statistics::*;
+pub use self::slice_statistics::*;
 pub use self::statistics::*;
 pub use self::traits::*;
 
-mod traits;
-mod statistics;
-mod order_statistics;
 mod iter_statistics;
+mod order_statistics;
+// TODO: fix later
 mod slice_statistics;
+mod statistics;
+mod traits;
